/* Beholder RMON ethernet network monitor,Copyright (C) 1993 DNPAP group */
/* See file COPYING 'GNU General Public Licence' for copyright details   */
#define INCL_BASE
#ifdef OS2
	#include <os2.h>
#endif
#include <fcntl.h>
#include <memory.h>
#include <string.h>
#include <stdio.h>
#include <maxmem.h>
#include <dnpap.h>
#include <message.h>
#include <timer.h>
#include <block.h>
#include <mac.h>

#include <ether.h>
#ifdef OS2
	#include <snor.h>
	#include <pipe.h>
	#include <soft.h>
	#include <loop.h>
	#include <echod.h>
	#include <pingd.h>
#endif
#ifdef SOLARIS
	#include <dlpi.h>
#endif
#ifdef SUNOS
	#include <dkvm.h>
	#include <nit.h>
#endif
#ifdef ULTRIX
	#include <dkvm.h>
	#include <pf.h>
#endif
#ifdef LINUX
	#include <eth.h>
#endif

#include <agent.h>
#include <ip.h>
#include <icmp.h>
#include <udp.h>
#include <snmp.h>
#include <mibsup.h>
#include <sysmib.h>
#include <ifmib.h>
#include <snmpmib.h>
#include <maxmib.h>
#include <agent.h>
#include <info.h>

#include <stat.h>
#include <hist.h>
#include <protocol.h>
#include <capture.h>
#include <filter.h>
#include <channel.h>
#include <matrix.h>
#include <host.h>
#include <topn.h>
#include <alarm.h>
#include <event.h>
#include <config.h>
#include <gen.h>
#include <reach.h>

#ifdef PERFORMANCE_TEST
#include <perf.h>
#include <perfmib.h>
#endif

#include <register.h>
#include <statbuf.h>

#ifdef DISCOVERY
#include <discnod.h>
#include <disccon.h>
#endif

#ifdef PACKETTYPE
#include <ptype.h>
#endif

#ifdef DNPAP_RMON_EXTENSIONS
#include <dstat.h>
#include <dfilter.h>
#include <dchan.h>
#include <dcapt.h>
#include <dhost.h>
#include <dtopn.h>
#include <dmatrix.h>
#include <devent.h>
#endif

#ifdef NEURO
#include <neuro.h>
#endif

#include "beholder.h"


BOOLEAN BeholderStrictRMON;


#if defined(SOLARIS)
	kstat_ctl_t *kstat = NULL;
	kid_t        kstatid = 0;
#endif
#if defined(SUNOS) || defined(ULTRIX)
	Kvm *kvm = NULL;
#endif
    
              
BYTE *trapComm = NULL;
USHORT trapCommLen = 0;


MIB_ELEMENT beholderObj[] =
{
#include "sys.mib"
#include "if.mib"
#include "snmp.mib"
#include "rmon.mib"
#include "info.mib"

#ifdef PACKETTYPE
#include "ptype.mib"
#endif

#ifdef DNPAP_RMON_EXTENSIONS
#include "statbuf.mib"
#endif	

#ifdef PERFORMANCE_TEST
#include "perf.mib"
#endif

#include "memory.mib"
#include "register.mib"

#ifdef NEURO
#include "neuro.mib"
#endif

#include "reach.mib"

#ifdef DISCOVERY
#include "disc.mib"
#endif

#ifdef DNPAP_RMON_EXTENSIONS
#include "drmon.mib"
#endif
};


static VOID BeholderSignalHandler(INT signum);



INT main(INT argc, char **argv)
{
    if (ConfigInit(argc, argv) == FALSE || ConfigLoad() == FALSE)
    {
        DnpapMessage(DMC_FATAL, BEHOLDER_CONFIGFAIL, "ConfigInit failed");
        DnpapExit(1);
    }

    MemoryInit(8000000L, 32000L);

    if (DnpapSignal(SIGINT, BeholderSignalHandler) == SIG_ERR)
        DnpapMessage(DMC_FATAL, BEHOLDER_SIGNALFAIL, "Signal handler initialization failed");
    
    MessageInit("stdio:out", MSGFILE, MSGSTDIO, NULL);
    MessageConfig(BEHOLDER_ERROR, "Beholder");
    
    if (ConfigGetBoolean("beholder.rmon.strict", &BeholderStrictRMON) == FALSE)
		BeholderStrictRMON = FALSE;

#if defined(SOLARIS)
	if ((kstat = kstat_open()) == NULL || (kstatid = kstat->kc_chain_id, FALSE))
		DnpapMessage(DMC_FATAL, BEHOLDER_KVMFAIL, "Open of kernel statistics memory failed");
#endif
#if defined(SUNOS) || defined(ULTRIX)
	if ((kvm = KvmOpen(O_RDONLY)) == NULL)
		DnpapMessage(DMC_FATAL, BEHOLDER_KVMFAIL, "Open of kernel memory failed");
#endif

    if (!TimerInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_TIMERFAIL,"TimerInit failed");

    /*  must come before PipeInit & SnorInit  */
#ifdef OS2
    if (!BlockInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_BLOCKFAIL,"BlockInit failed");
#else	
    if (BlockInit() != 0)
        DnpapMessage(DMC_WARNING,BEHOLDER_BLOCKFAIL,"BlockInit failed");
#endif		

    if (!ProtInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_PROTFAIL,"ProtInit failed");
    
#ifdef OS2
    if (!PipeInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_PIPEFAIL,"PipeInit failed");
#endif /* OS2 */

    /*  must come before MacInit  */
#ifdef OS2	
    if (!SnorInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_SNORFAIL,"SnorInit failed");
#else
	if (!NwInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_SNORFAIL,"NwInit failed");
#ifdef SOLARIS
	if (!DlpiInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_SNORFAIL,"DlpiInit failed");
#endif
#ifdef SUNOS
	if (!NitInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_SNORFAIL,"NitInit failed");
#endif
#ifdef ULTRIX
	if (!PfInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_SNORFAIL,"PfInit failed");
#endif
#ifdef LINUX
	if (!EthInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_SNORFAIL,"EthInit failed");
#endif
#endif

#ifdef OS2	
    /*  must come before MacInit  */
    if (!LoopInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_LOOPFAIL,"LoopInit failed");
#endif
    
    if (!MacInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_MACFAIL,"MacInit failed");

    /*  must come before MibInit  */
    if (!AgentInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_AGENTFAIL,"AgentInit failed");
		
    /*  must come after AgentInit  */
    if (!MibInit(beholderObj, sizeof(beholderObj)/sizeof(beholderObj[0])))
        DnpapMessage(DMC_WARNING,BEHOLDER_MIBFAIL,"MibInit failed");

#ifdef OS2    
    if (!ArpInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_ARPFAIL,"ArpInit failed");
    
    if (!IpInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_IPFAIL,"IpInit failed");
#endif
    
    if (!IcmpInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_ICMPFAIL,"IcmpInit failed");
    
    if (!UdpInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_UDPFAIL,"UdpInit failed");
		
#ifdef OS2
    if (!EchodInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_ECHOFAIL,"EchodInit failed");

    if (!PingdInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_PINGFAIL,"PingdInit failed");
#endif /* OS2 */

    if (!EtherInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_ETHERFAIL,"EtherInit failed");
   
#ifdef OS2
    if (!SoftInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_SOFTFAIL,"SoftInit failed");
#endif /* OS2 */
     
    if (!GenInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_GENFAIL,"GenInit failed");
    
    if (!SysInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_SYSFAIL,"SysInit failed");

    /*  requests info on iface's, so place it at least after MacInit  */
    if (!EtherStatsMInit())   
        DnpapMessage(DMC_WARNING,BEHOLDER_STATSFAIL,"EtherStatsMInit failed");
    
    /*  requests info on iface's, so place it at least after MacInit  */
    if (!HistoryMInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_HISTORYFAIL,"HistoryMInit failed");
    
    /*  requests info on iface's, so place it at least after MacInit  */
    if (!HostMInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_HOSTFAIL,"HostMInit failed");
    
    if (!TopNMInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_TOPNFAIL,"TopNMInit failed");
    
    /*  requests info on iface's, so place it at least after MacInit  */
    if (!MatrixMInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_MATRIXFAIL,"MatrixMInit failed");
    
    if (!EventMInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_EVENTFAIL,"EventMInit failed");
    
    if (!AlarmMInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_ALARMFAIL,"AlarmMInit failed");
    
    if (!ChannelMInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_CHANNELFAIL,"ChannelMInit failed");
    
    if (!CaptureMInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_CAPTUREFAIL,"CaptureMInit failed");
    
    if (!FilterMInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_FILTERFAIL,"FilterMInit failed");

    /*  requests info on iface's, so place it at least after MacInit  */
    if (!InfoMInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_INFOFAIL,"InfoMInit failed");

#ifdef PACKETTYPE
    /*  requests info on iface's, so place it at least after MacInit  */
    if (!PacketTypeMInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_TYPEFAIL,"PacketTypeMInit failed");
#endif

#ifdef PERFORMANCE_TEST
    if (!PerfInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_PERFFAIL,"PerfInit failed");
    
    if (!PerfMInit())   /*  MIB support initialization for perf  */
        DnpapMessage(DMC_WARNING,BEHOLDER_PERFMFAIL,"PerfMInit failed");
#endif
    
    if (!StatBufMInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_STATBUFMFAIL,"StatBufMInit failed");

#ifdef NEURO
    if (!NeuroMInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_NEUROMFAIL,"NeuroMInit failed");
#endif

    if (!ReachMInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_REACHMFAIL,"ReachMInit failed");

#ifdef DISCOVERY   
    if (!DiscNodeMInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_DISCNODEMFAIL,"DiscNodeMInit failed");

    if (!DiscConnMInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_DISCCONNMFAIL,"DiscConnMInit failed");
#endif

#ifdef DNPAP_RMON_EXTENSIONS
    if (!DnpapEtherStatsMInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_DNPAPSTATSFAIL,"DnpapEtherStatsMInit failed");
    
    if (!DnpapHostMInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_DNPAPHOSTFAIL,"DnpapHostMInit failed");

    if (!DnpapTopNMInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_DNPAPTOPNFAIL,"DnpapTopNMInit failed");
    
    if (!DnpapMatrixMInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_DNPAPMATRIXFAIL,"DnpapMatrixMInit failed");

    if (!DnpapEventMInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_DNPAPEVENTFAIL,"DnpapEventMInit failed");

    if (!DnpapChannelMInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_DNPAPCHANFAIL,"DnpapChannelMInit failed");

    if (!DnpapCaptureMInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_DNPAPCAPTFAIL,"DnpapCaptureMInit failed");

    if (!DnpapFilterMInit())
        DnpapMessage(DMC_WARNING,BEHOLDER_DNPAPFILTERFAIL,"DnpapFilterMInit failed");
#endif

    if (ConfigGetBytes("beholder.trap.community", &trapComm, &trapCommLen) == FALSE)
	{
        trapComm = "trap";
        trapCommLen = 4;
	}
    if (!AgentSendTrap(trapComm, trapCommLen, SNMP_TRAP_COLDSTART, 0, 0, 0))
        DnpapMessage(DMC_WARNING,BEHOLDER_TRAPFAIL,"Could not send COLDSTART trap");

    DnpapMessage(DMC_MESSAGE,BEHOLDER_ACTIVE,"Beholder TNG active !\n");
 
#ifdef OS2    
    if (!Block())
        DnpapMessage(DMC_WARNING,BEHOLDER_BLOCKERR,"Block exit !");
#else            
    while (1)
        if (Block()!=0)
            DnpapMessage(DMC_WARNING,BEHOLDER_BLOCKERR,"Block exit !");
#endif
        
	DnpapExit(0);

    return 0;
}


VOID BeholderSignalHandler(INT signum)
{
#if defined(SOLARIS)
	kstat_close(kstat);
#endif
#if defined(SUNOS) || defined(ULTRIX)
	KvmClose(kvm);
#endif

    DnpapExit(0);
}

